using System;
using System.Collections;
using System.Text;
using System.Web.UI;
using System.IO;
using System.Xml;
using System.Xml.XPath;

namespace RussellMason.Web.UI.Office2007
{

	/// <summary>
	/// Provides serialization and desialization methods for Ribbon based controls.
	/// </summary>
	public class RibbonSerializer
	{

		#region Fileds

		private static RibbonSerializer _default = null;

		#endregion Fileds

		#region Static Properties

		/// <summary>
		/// Gets or sets an instance of the RibbonSelializer that is always available.
		/// This allows the serialization to be customized by deriving from RibbonSerializer
		/// without requiring any changes to the source caller.
		/// </summary>
		public static RibbonSerializer Default
		{
			get
			{
				if (_default == null)
				{
					_default = new RibbonSerializer();
				}

				return _default;
			}
			set
			{
				if (value == null)
				{
					throw new ArgumentNullException("Default");
				}

				_default = value;
			}
		} 

		#endregion Static Properties

		#region Static Methods

		/// <summary>
		/// Stores the state of ribbon controls in the specified page in a Session
		/// variable called 'RibbonState' allowing it to survive page redirects.
		/// </summary>
		/// <param name="page">
		/// The page to store the ribbon state of.
		/// </param>
		public static void StorePageRibbonState(Control page)
		{
			string xml = RibbonSerializer.Default.Serialize(page);

			page.Page.Session["RibbonState"] = xml;
		}

		/// <summary>
		/// Restores the state of ribbon controls in the specified page from a Session
		/// variable called 'RibbonState'. Clears the Session variable once restored.
		/// </summary>
		/// <param name="control">
		/// The page to restore the ribbon state to.
		/// </param>
		public static void RestorePageRibbonState(Control page)
		{
			RestorePageRibbonState(page, true);
		}

		/// <summary>
		/// Restores the state of ribbon controls in the specified page from a Session
		/// variable called 'RibbonState'.
		/// </summary>
		/// <param name="control">
		/// The page to restore the ribbon state to.
		/// </param>
		public static void RestorePageRibbonState(Control page, bool clearWhenComplete)
		{
			string xml = page.Page.Session["RibbonState"] as string;

			try
			{
				if (!StringUtil.IsNullOrEmpty(xml))
				{
					RibbonSerializer.Default.Deserialize(page, xml);
				}
			}
			finally
			{
				if (clearWhenComplete)
				{
					page.Page.Session.Remove("RibbonState");
				}
			}
		} 

		#endregion Static Methods

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the RibbonSerializer class.
		/// </summary>
		public RibbonSerializer()
		{
		} 

		#endregion Constructors

		#region Methods

		/// <summary>
		/// Serializes the state of ribbon based controls. This is intended for restoring 
		/// state rather than creating control instances.
		/// <para />
		/// The resulting XML is linear, i.e. all controls are recoded as a single list
		/// rather than a hierarchy.
		/// </summary>
		/// <param name="control">
		/// The control to start the serialization with. Only controls within this control 
		/// are serialized. This may be a Page.
		/// </param>
		/// <returns>
		/// The xml containing the serialized state.
		/// </returns>
		public virtual string Serialize(Control control)
		{
			StringBuilder builder = new StringBuilder();

//			XmlWriterSettings settings = new XmlWriterSettings();
//			settings.Indent = true;


			System.IO.StringWriter stringWriter = new System.IO.StringWriter(builder);
			XmlTextWriter writer = new System.Xml.XmlTextWriter(stringWriter);
			writer.Indentation = 5;

			writer.WriteStartElement("RibbonControls");
			writer.WriteAttributeString("Url", control.Page.Request.Url.ToString());

			Serialize(writer, control);

			writer.WriteEndElement();

			writer.Close();

			string xml = builder.ToString();

			return xml;
		}

		/// <summary>
		/// Deserializes the state of ribbon based controls. This does not create control instances
		/// but matches controls and sets the appropriate property values against them.
		/// Allows controls to be deserialized accross page boundaries using the UniqueID for 
		/// matching.
		/// </summary>
		/// <param name="control">
		/// The control to start the deserialization with. Only controls within this 
		/// control are deserialized. This may be a Page.
		/// </param>
		/// <param name="xml">
		/// The xml containing the serialized state.
		/// </param>
		public virtual void Deserialize(Control control, string xml)
		{
			Deserialize(control, xml, DeserializationPageMatch.Any, DeserializationControlMatch.UniqueID);
		} 

		/// <summary>
		/// Deserializes the state of ribbon based controls. This does not create control instances
		/// but matches controls and sets the appropriate property values against them.
		/// </summary>
		/// <param name="control">
		/// The control to start the deserialization with. Only controls within this 
		/// control are deserialized. This may be a Page.
		/// </param>
		/// <param name="xml">
		/// The xml containing the serialized state.
		/// </param>
		/// <param name="pageMatch">
		/// Incidates whether state can be restored to controls on a page other than that used
		/// to serialize it.
		/// </param>
		/// <param name="controlMatch">
		/// Incidates how a control is matched with the serialized information.
		/// </param>
		public virtual void Deserialize(Control control, string xml, DeserializationPageMatch pageMatch,
			DeserializationControlMatch controlMatch)
		{
			XmlDocument document = new XmlDocument();
			document.LoadXml(xml);

			// Gets root node and check page URL againt match type
			XmlElement element = document.SelectSingleNode("RibbonControls") as XmlElement;

			if (element != null)
			{
				Uri pageUri = new Uri(element.Attributes["Url"].Value);
				Uri requestUri = control.Page.Request.Url;

				string pageUrl = pageUri.AbsoluteUri;
				string requestUrl = requestUri.AbsoluteUri;

				if (!StringUtil.IsNullOrEmpty(pageUri.Query))
				{
					pageUrl = pageUrl.Substring(0, pageUrl.Length - pageUri.Query.Length);
				}

				if (!StringUtil.IsNullOrEmpty(requestUri.Query))
				{
					requestUrl = requestUrl.Substring(0, requestUrl.Length - requestUri.Query.Length);
				}

				bool matchesPage = ((pageMatch == DeserializationPageMatch.Any) ||
					((pageMatch == DeserializationPageMatch.PageUrl) && (pageUrl == requestUrl)) ||
					((pageMatch == DeserializationPageMatch.FullUrl) && (pageUri.ToString() == requestUri.ToString())));

				if (matchesPage)
				{
					Deserialize(element, control, controlMatch);
				}
			}
		} 

		#endregion Methods

		#region Implementation Methods

		/// <summary>
		/// Serializes the state of ribbon based controls. You should serialize the control only,
		/// do not include child controls, they will be serialized as part of the process.
		/// </summary>
		/// <param name="writer">
		/// The writer to write XML to.
		/// </param>
		/// <returns>
		/// The control to serialize.
		/// </returns>
		protected virtual void Serialize(XmlWriter writer, Control control)
		{
			RibbonControl ribbonControl = control as RibbonControl;

			if (ribbonControl != null)
			{
				ribbonControl.Serialize(writer);
			}

			foreach (Control childControl in control.Controls)
			{
				Serialize(writer, childControl);
			}
		}

		/// <summary>
		/// Deserializes the state of ribbon based controls. You should deserialize the control only,
		/// do not include child controls, they will be deserialized as part of the process.
		/// </summary>
		/// <param name="element">
		/// The root element containing a list of serialized controls.
		/// </param>
		/// <param name="control">
		/// The control to deserialize into.
		/// </param>
		/// <param name="controlMatch">
		/// Incidates how a control is matched with the serialized information.
		/// </param>
		protected virtual void Deserialize(XmlElement element, Control control, DeserializationControlMatch controlMatch)
		{
			RibbonControl ribbonControl = control as RibbonControl;

			if (ribbonControl != null)
			{
				string matchId = control.UniqueID;
				string xpath = string.Format("RibbonControl[@UniqueID='{0}']", matchId);

				if (controlMatch == DeserializationControlMatch.ID)
				{
					matchId = control.ID;
					xpath = string.Format("RibbonControl[@ID='{0}']", matchId);
				}

				XmlElement controlElement = element.SelectSingleNode(xpath) as XmlElement;

				if (controlElement != null)
				{
					string type = controlElement.Attributes["Type"].Value;

					if (type == control.GetType().Name)
					{
						ribbonControl.Deserialize(controlElement);
					}
				}
			}

			foreach (Control childControl in control.Controls)
			{
			  Deserialize(element, childControl, controlMatch);
			}
		}  

		#endregion Implementation Methods

	}

}
