using System;
using System.Collections;
using System.Text;
using System.Web.UI.HtmlControls;
using System.ComponentModel;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Drawing;

//using RussellMason.Web.UI.Office2007.Design;

namespace RussellMason.Web.UI.Office2007
{

	/// <summary>
	/// Represents a small button for use on the Ribbon Bar.
	/// </summary>
	[
	//Designer(typeof(SmallRibbonButtonDesigner)),
	ToolboxData("<{0}:SmallRibbonButton runat=\"server\"></{0}:SmallRibbonButton>")
	]
	public class SmallRibbonButton : TextRibbonButton
	{

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the SmallRibbonButton class.
		/// </summary>
		public SmallRibbonButton()
			: this(string.Empty)
		{
			// Shared State
			// 0 - Enabled
			// 1 - Checked
			// 2 - EnabledImageUrl
			// 3 - DisabledImageUrl
			// 4 - Text
			// 5 - DropDownMenuID
		}

		/// <summary>
		/// Initializes a new instance of the SmallRibbonButton class.
		/// </summary>
		/// <param name="id">
		/// The ID to assign to the control.
		/// </param>
		public SmallRibbonButton(string id)
			: this(id, string.Empty)
		{
		}

		/// <summary>
		/// Initializes a new instance of the SmallRibbonButton class.
		/// </summary>
		/// <param name="id">
		/// The ID to assign to the control.
		/// </param>
		/// <param name="text">
		/// The button text.
		/// </param>
		public SmallRibbonButton(string id, string text)
			: this(id, text, string.Empty, string.Empty)
		{
		}

		/// <summary>
		/// Initializes a new instance of the SmallRibbonButton class.
		/// </summary>
		/// <param name="id">
		/// The ID to assign to the control.
		/// </param>
		/// <param name="text">
		/// The button text.
		/// </param>
		/// <param name="enabledImageUrl">
		/// The URL to an image to display when enabled.
		/// </param>
		/// <param name="disabledImageUrl">
		/// The URL to an image to display when disabled.
		/// </param>
		public SmallRibbonButton(string id, string text, string enabledImageUrl, string disabledImageUrl)
		{
			this.ID = id;
			this.Text = text;
			this.EnabledImageUrl = enabledImageUrl;
			this.DisabledImageUrl = disabledImageUrl;

			// Default Width;
			this.Width = new Unit("45px");
		}

		#endregion Constructors

		#region Methods

		/// <summary>
		/// Automatically sets the width of the button to match the button's text. Sets to a 
		/// minimum width of 45 pixels. This is for internal use to set defaults. A small 
		/// ribbon button should always use a signle line.
		/// </summary>
		/// <param name="allowWrap">
		/// True to allow the text to wrap on to 2 lines, false to force the text on a single 
		/// line.
		/// </param>
		public override void AutoSizeWidth(bool allowWrap)
		{
			Font font = new Font("Arial", 8.0f);

			AutoSizeWidth(font, 13, 22, allowWrap);
		} 

		#endregion Methods

		#region Implementation Methods

		/// <summary>
		/// Creates the control's layout.
		/// </summary>
		protected override void CreateChildControls()
		{
			ClearControls();

			this.Attributes.Add("class", "smallRibbonButton");
			this.Attributes.Add("title", this.ToolTip);

			HtmlGenericControl startDiv = new HtmlGenericControl("div");
			startDiv.EnableViewState = false;
			startDiv.ID = "Start";

			this.Controls.Add(startDiv);

			HtmlGenericControl itemDiv = new HtmlGenericControl("div");
			itemDiv.EnableViewState = false;
			itemDiv.ID = "Item";

			this.Controls.Add(itemDiv);

			HtmlGenericControl imageDiv = new HtmlGenericControl("div");
			imageDiv.EnableViewState = false;
			imageDiv.Attributes.Add("class", "smallRibbonButtonImage");

			itemDiv.Controls.Add(imageDiv);

			HtmlImage image = new HtmlImage();
			image.EnableViewState = false;
			image.ID = "Image";

			imageDiv.Controls.Add(image);

			HtmlGenericControl textDiv = new HtmlGenericControl("div");
			textDiv.EnableViewState = false;
			textDiv.ID = "Text";
			textDiv.Attributes.Add("class", "smallRibbonButtonText");

			itemDiv.Controls.Add(textDiv);

			HtmlGenericControl dropDownDiv = new HtmlGenericControl("div");
			dropDownDiv.EnableViewState = false;
			dropDownDiv.ID = "DropDownMenuImage";

			itemDiv.Controls.Add(dropDownDiv);

			HtmlGenericControl endDiv = new HtmlGenericControl("div");
			endDiv.EnableViewState = false;
			endDiv.ID = "End";

			this.Controls.Add(endDiv);
		}

		/// <summary>
		/// Sets the internal width of the button.
		/// </summary>
		protected override void SetWidth()
		{
			int innerWidth = 0;

			if (this.Width.Value > 4)
			{
				innerWidth = Convert.ToInt32(this.Width.Value) - 4;
			}

			HtmlGenericControl itemDiv = FindControl("Item") as HtmlGenericControl;

			itemDiv.Style.Add("width", innerWidth.ToString() + "px");
		}

		/// <summary>
		/// Sets the button's checked state.
		/// </summary>
		protected override void SetCheckedState()
		{
			HtmlGenericControl startDiv = FindControl("Start") as HtmlGenericControl;
			HtmlGenericControl itemDiv = FindControl("Item") as HtmlGenericControl;
			HtmlGenericControl endDiv = FindControl("End") as HtmlGenericControl;

			if (this.Checked)
			{
				startDiv.Attributes.Add("class", "smallRibbonButtonStartChecked");
				itemDiv.Attributes.Add("class", "smallRibbonButtonItemChecked");
				endDiv.Attributes.Add("class", "smallRibbonButtonEndChecked");
			}
			else
			{
				startDiv.Attributes.Add("class", "smallRibbonButtonStart");
				itemDiv.Attributes.Add("class", "smallRibbonButtonItem");
				endDiv.Attributes.Add("class", "smallRibbonButtonEnd");
			}
		}

		/// <summary>
		/// Sets the button's text.
		/// </summary>
		protected override void SetText()
		{
			HtmlGenericControl div = FindControl("Text") as HtmlGenericControl;

			if (div != null)
			{
				if (this.Enabled)
				{
					div.Attributes.Add("class", "smallRibbonButtonText");
				}
				else
				{
					div.Attributes.Add("class", "smallRibbonButtonTextDisabled");
				}

				div.InnerText = this.Text;
			}
		}

		/// <summary>
		/// Sets whether the drop down image is visible.
		/// </summary>
		protected override void SetDropDownMenu()
		{
			HtmlGenericControl dropDownDiv =
				FindControl("DropDownMenuImage") as HtmlGenericControl;

			if (dropDownDiv != null)
			{
				if (this.Enabled)
				{
					dropDownDiv.Attributes.Add("class", "smallRibbonButtonDropDownMenuArrow");
				}
				else
				{
					dropDownDiv.Attributes.Add("class", "smallRibbonButtonDropDownMenuArrowDisabled");
				}

				dropDownDiv.Visible = (!StringUtil.IsNullOrEmpty(this.DropDownMenuID));
			}
		}

		#endregion Implementation Methods

	}

}
