using System;
using System.Collections;
using System.Text;
using System.Drawing;
using System.ComponentModel;
using System.Xml;

namespace RussellMason.Web.UI.Office2007
{

	/// <summary>
	/// Base class for Ribbon buttons with text.
	/// </summary>
	public class TextRibbonButton : RibbonButton
	{

		#region Fields

		private string _dropDownMenuID = string.Empty;

		#endregion Fields

		#region Constructors

		/// <summary>
		/// Allows derived classes to create an instance of the TextRibbonButton class.
		/// </summary>
		protected TextRibbonButton()
		{
			// Shared State
			// 0 - Enabled
			// 1 - Checked
			// 2 - EnabledImageUrl
			// 3 - DisabledImageUrl
			// 4 - Text
			// 5 - DropDownMenuID
			this.SharedState.Add(new SharedStateItem("Text", typeof(string), string.Empty));
			this.SharedState.Add(new SharedStateItem("DropDownMenuID", typeof(string), string.Empty));
		} 

		#endregion Constructors

		#region Properties

		/// <summary>
		/// Gets or sets the text to display in the button.
		/// </summary>
		[
		DefaultValue(""),
		Description("Specifies the text to display in the button."),
		Category("Appearance")
		]
		public virtual string Text
		{
			get
			{
				string text = Convert.ToString(this.SharedState["Text"].Value);

				if (text == null)
				{
					text = string.Empty;
				}

				return text;
			}
			set
			{
				this.SharedState["Text"].Value = value;
			}
		}

		/// <summary>
		/// Gets or sets the ID of the drop down menu associated with the button. 
		/// N.B. On the server this is the control's ID, on the client this is the control's ClientID. 
		/// Empty if not set.
		/// </summary>
		[
		DefaultValue(null),
		Description("Specifies the ID of the drop down menu associated with the button."),
		Category("Behavior")
		]
		public virtual string DropDownMenuID
		{
			get
			{
				return _dropDownMenuID;
			}
			set
			{
				// The client ID will be calculated when shared state is saved as the associated
				// control may not have been loaded yet
				_dropDownMenuID = value;
			}
		}

		#endregion Properties

		#region Methods

		/// <summary>
		/// Sets text - allows use by control and designer.
		/// </summary>
		public override void FinalizeRender()
		{
			base.FinalizeRender();

			SetWidth();
			SetText();
			SetDropDownMenu();
		}

		/// <summary>
		/// Automatically sets the width of the button to match the button's text. Sets to a
		/// minimum width of 45 pixels on a single line.
		/// </summary>
		public virtual void AutoSizeWidth()
		{
			AutoSizeWidth(false);
		}

		/// <summary>
		/// Automatically sets the width of the button to match the button's text. Sets to a 
		/// minimum width of 45 pixels.
		/// </summary>
		/// <param name="allowWrap">
		/// True to allow the text to wrap on to 2 lines, false to force the text on a single 
		/// line.
		/// </param>
		public virtual void AutoSizeWidth(bool allowWrap)
		{
			Font font = new Font("Arial", 8.0f);

			AutoSizeWidth(font, 4, 45, allowWrap);
		}

		/// <summary>
		/// Automatically sets the width of the button to match the button's text.
		/// </summary>
		/// <param name="font">
		/// The font to base the size on.
		/// </param>
		/// <param name="padding">
		/// Additional padding left and right.
		/// </param>
		/// <param name="minWidth">
		/// The minimum width to use.
		/// </param>
		/// <param name="allowWrap">
		/// True to allow the text to wrap on to 2 lines, false to force the text on a single 
		/// line.
		/// </param>
		public virtual void AutoSizeWidth(Font font, int padding, int minWidth, bool allowWrap)
		{
			Size size = Size.Empty;

			if (allowWrap)
			{
				size = MeasureWrappedString(this.Text, font);
			}
			else
			{
				size = MeasureString(this.Text, font);
			}

			// Allow for arrow image
			if (!StringUtil.IsNullOrEmpty(this.DropDownMenuID)) 
			{
				padding += 5;
			}

			int width = Math.Max(size.Width + (padding * 2), minWidth);

			this.Width = width;
		} 

		#endregion Methods

		#region Implementation Methods

		/// <summary>
		/// Sets the originally selected tab prior to saving shared state.
		/// </summary>
		protected override void SaveSharedState()
		{
			// Calculate before saving state as the control may not be available when the 
			// property was set
			string clientId = GetClientID(this.Page, this.DropDownMenuID);
			this.SharedState["DropDownMenuID"].Value = clientId;

			base.SaveSharedState();
		}

		/// <summary>
		/// Sets the internal width of the button.
		/// </summary>
		protected virtual void SetWidth()
		{
			// Not set by default but allows derived classes to if they need to
		}

		/// <summary>
		/// Sets the button's text.
		/// </summary>
		protected virtual void SetText()
		{
			// Not set by default but allows derived classes to if they need to
		}

		/// <summary>
		/// Sets whether the drop down image is visible.
		/// </summary>
		protected virtual void SetDropDownMenu()
		{
			// Not set by default but allows derived classes to if they need to
		}

		/// <summary>
		/// Writes the object's details to the specified writer.
		/// </summary>
		/// <param name="writer">
		/// The writer to write to.
		/// </param>
		protected override void SerializeContent(XmlWriter writer)
		{
			base.SerializeContent(writer);

			writer.WriteAttributeString("Text", this.Text);
			writer.WriteAttributeString("DropDownMenuID", this.DropDownMenuID);
		}

		/// <summary>
		/// Deserializes the object from the specified node.
		/// </summary>
		/// <param name="element">
		/// The element to get property values from.
		/// </param>
		public override void Deserialize(XmlElement element)
		{
			base.Deserialize(element);

			if (element.HasAttribute("Text"))
			{
				string text = element.Attributes["Text"].Value;
				this.Text = text;
			}

			if (element.HasAttribute("DropDownMenuID"))
			{
				string dropDownMenuID = element.Attributes["DropDownMenuID"].Value;
				this.DropDownMenuID = dropDownMenuID;
			}
		}

		#endregion Implementation Methods

	}

}
