using System;
using System.Collections;
using System.Text;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Text.RegularExpressions;

namespace RussellMason.Web.UI
{

	/// <summary>
	/// Container for holding state that can be shared on the server though ASP and 
	/// client through java script.
	/// </summary>
	public class SharedStateBag
	{

		#region Fields

		private Control _parent = null;
		private IList _items = null; 

		#endregion Fields

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the SharedStateBag class. 
		/// </summary>
		/// <param name="parent">
		/// The control that owns the state bag.
		/// </param>
		public SharedStateBag(Control parent)
		{
			if (parent == null)
			{
				throw new ArgumentNullException("parent");
			}

			_parent = parent;
		} 

		#endregion Constructors

		#region Properties

		/// <summary>
		/// Gets the control the state is associated with.
		/// </summary>
		public Control Parent
		{
			get
			{
				return _parent;
			}
		}

		/// <summary>
		/// Gets the ID of the hidden field used to store the state (Allways 'SharedState').
		/// </summary>
		public string SharedStateFieldID
		{
			get
			{
				return "SharedState";
			}
		}

		/// <summary>
		/// Gets the value at the specified index.
		/// </summary>
		/// <param name="index">
		/// The index of the value within the list.
		/// </param>
		/// <returns>
		/// The value if available; otherwise null.
		/// </returns>
		public SharedStateItem this[int index]
		{
			get
			{
				return (SharedStateItem)this.Items[index];
			}
		}

		/// <summary>
		/// Gets the value with the specified key. If the key is null or string.Emtpy
		/// null is return irrespective of empty keys in the bag. If an item does not
		/// exist with the key null is returned. Keys are case-sensitive.
		/// </summary>
		/// <param name="key">
		/// The key to find within the list.
		/// </param>
		/// <returns>
		/// The value if available; otherwise null.
		/// </returns>
		public SharedStateItem this[string key]
		{
			get
			{
				SharedStateItem item = null;
				int index = IndexOf(key);

				if (index != -1)
				{
					item = (SharedStateItem)this.Items[index];
				}

				return item;
			}
		}

		/// <summary>
		/// Gets the number of items in the bag.
		/// </summary>
		public int Count
		{
			get
			{
				return this.Items.Count;
			}
		}

		#endregion Properties

		#region Methods

		/// <summary>
		/// Clears the shared state bag.
		/// </summary>
		public void Clear()
		{
			this.Items.Clear();
		}

		/// <summary>
		/// Add the specified item to the bag.
		/// </summary>
		/// <param name="item">
		/// The item to add.
		/// </param>
		public void Add(SharedStateItem item)
		{
			int index = IndexOf(item.Key);

			if (index != -1)
			{
				string message =
					string.Format("Item with key '{0}' already exists within the SharedStateBag.", item.Key);

				throw new Exception(message);
			}

			this.Items.Add(item);
		}

		/// <summary>
		/// Removes the item with the specified key from the bag.
		/// Keys are case-sensitive.
		/// </summary>
		/// <param name="key">
		/// The key of the item to remove.
		/// </param>
		public void Remove(string key)
		{
			int index = IndexOf(key);

			if (index != -1)
			{
				RemoveAt(index);
			}
		}

		/// <summary>
		/// Removes the item at the specified index.
		/// </summary>
		/// <param name="index">
		/// The index of the item to remove.
		/// </param>
		public void RemoveAt(int index)
		{
			this.Items.RemoveAt(index);
		}

		/// <summary>
		/// Gets the index of the specified key. Returns -1 if not found.
		/// </summary>
		/// <param name="key">
		/// The key to find. Keys are case-sensitive.
		/// </param>
		/// <returns>
		/// The index if available; otherwise -1.
		/// </returns>
		public int IndexOf(string key)
		{
			int foundIndex = -1;

			if (!StringUtil.IsNullOrEmpty(key))
			{
				for (int index = 0; index < this.Count; index++)
				{
					SharedStateItem item = this[index];

					if (item.Key == key)
					{
						foundIndex = index;
						break;
					}
				}
			}

			return foundIndex;
		}

		/// <summary>
		/// Loads the state based on the items within the bag.
		/// In the format '[item,item,item]'. If the item is a string it must be enclosed in 
		/// single quotes. Items must not have white space between them, e.g. [10,'MyValue',20,true,'Done'].
		/// </summary>
		public virtual void Load()
		{
			if (this.StateField != null)
			{
				string state = this.StateField.Value.Trim().Trim('[', ']');

				if (!StringUtil.IsNullOrEmpty(state))
				{
					// Split at commas (not if inside quotes)
					string[] values = SplitState(state);

					for (int index = 0; index < values.Length; index++)
					{
						SharedStateItem item = (SharedStateItem)this.Items[index];

						state = RemoveQuotes(values[index]);

						if (!StringUtil.IsNullOrEmpty(state))
						{
							object value = DeserializeState(item.Type, state);

							item.Value = value;
						}
					}
				}
			}
		}

		/// <summary>
		/// Saves a string representation of the state to a hidden field in the parent control. 
		/// In the format '[item,item,item]'. If the item is a string it is enclosed in single quotes. 
		/// Items have no white space between them, e.g. [10,'MyValue',20,true,'Done'].
		/// </summary>
		public virtual void Save()
		{
			if (this.StateField != null)
			{
				this.StateField.Value = this.ToString();
			}
		}

		#endregion Methods

		#region Implementation Properties

		/// <summary>
		/// Gets the hidden field that will store the state.
		/// </summary>
		protected SharedStateHiddenField StateField
		{
			get
			{
				SharedStateHiddenField stateField = _parent.FindControl(this.SharedStateFieldID) as SharedStateHiddenField;

				return stateField;
			}
		}

		/// <summary>
		/// Gets a list of values for internal storage.
		/// </summary>
		protected IList Items
		{
			get
			{
				if (_items == null)
				{
					_items = new ArrayList();
				}

				return _items;
			}
		}

		#endregion Implementation Properties

		#region Implementation Methods

		/// <summary>
		/// Null is returned as an empty string. The value should have no starting or 
		/// trailing white space. String values or custom objects should be enclosed in
		/// single quotes.
		/// <para />
		/// In derived classes you can serialize special types or call base to use
		/// default serialization.
		/// </summary>
		/// <param name="value">
		/// The value to serialize.
		/// </param>
		/// <returns>
		/// The serialized value.
		/// </returns>
		protected virtual string SerializeState(object value)
		{
			string state = string.Empty;

			if (value != null)
			{
				Type valueType = value.GetType();

				if ((valueType == typeof(string)) || (valueType.IsEnum))
				{
					state = "'" + value.ToString().Replace("'", "\\'") + "'";
				}
				else if (valueType == typeof(bool))
				{
					state = value.ToString().ToLower();
				}
				else
				{
					state  = value.ToString();
				}
			}

			return state;
		}

		/// <summary>
		/// Deserializes the string value into an object of the specified type. By 
		/// default this simply converts the string to the type.
		/// <para />
		/// In derived classes you can deserialize special types or call base to use
		/// default serialization.
		/// </summary>
		/// <param name="type">
		/// The target type to deserialize in to.
		/// </param>
		/// <param name="value">
		/// The value to deserialize.
		/// </param>
		/// <returns>
		/// The deserialized object.
		/// </returns>
		protected virtual object DeserializeState(Type type, string value)
		{
			object item = null;

			if (value != null)
			{
				item = Convert.ChangeType(value, type);
			}

			return item;
		}

		/// <summary>
		/// Splits the specified value at commas (not including those in quoted strings).
		/// </summary>
		/// <param name="value">
		/// The value to split.
		/// </param>
		/// <returns>
		/// An array of state values.
		/// </returns>
		protected string[] SplitState(string value)
		{
			ArrayList list = new ArrayList();

			if (value != null)
			{
				string splitValue = string.Empty;
				int startIndex = 0;
				bool inQuotes = false;
				int index = 0;

				if (value.Length == 0)
				{
					list.Add(splitValue);
				}
				else
				{
					while (index < value.Length)
					{
						char checkChar = value[index];

						if ((checkChar == ',') && (!inQuotes))
						{
							splitValue = string.Empty;
							int length = index - startIndex;

							if (length > 0)
							{
								splitValue = value.Substring(startIndex, length);
							}

							list.Add(splitValue);

							startIndex = index + 1;  // Ready for next value
						}
						else if (checkChar == '\'')
						{
							bool escaped = false;

							if (index > 0)
							{
								char previousChar = value[index - 1];

								escaped = (previousChar == '\\');
							}

							if (!escaped)
							{
								inQuotes = !inQuotes;
							}
						}

						if (index >= value.Length - 1)
						{
							splitValue = string.Empty;
							int length = value.Length - startIndex;

							if (length > 0)
							{
								splitValue = value.Substring(startIndex, length);
							}

							list.Add(splitValue);
						}

						index++;
					}
				}
			}

			string[] items = new string[list.Count];
			int i=0;
			foreach(string item in list)
			{
				items[i] = item;
				i++;
			}
			return items;
		}

		/// <summary>
		/// Removes single quotes from the start and end of the value. Also
		/// changes escaped single quotes to just single quotes.
		/// </summary>
		/// <param name="value">
		/// The value to trim.
		/// </param>
		/// <returns>
		/// The trimmed string.
		/// </returns>
		protected string RemoveQuotes(string value)
		{
			if (value != null)
			{
				value = value.Trim().Trim('\'');

				value = value.Replace("\\'", "'");
			}

			return value;
		}

		/// <summary>
		/// Returns a string representation of the StateBag. This is the same as the 
		/// state string that is saved.
		/// </summary>
		/// <returns>
		/// The state of the bag.
		/// </returns>
		public override string ToString()
		{
			string state = string.Empty;

			if (this.Items.Count > 0)
			{
				state = "[";

				for (int index = 0; index < this.Items.Count; index++)
				{
					if (index > 0)
					{
						state += ",";
					}

					string itemState = SerializeState(((SharedStateItem)this.Items[index]).Value);

					state += itemState;
				}

				state += "]";
			}

			return state;
		}

		#endregion Implementation Methods

	}

}
