using System;
using System.Collections;
using System.Text;

namespace RussellMason.Web.UI
{

	/// <summary>
	/// Represents an item stored in the shared state bag.
	/// </summary>
	public class SharedStateItem
	{

		#region Fields

		private string _key = string.Empty;
		private Type _type = null;
		private object _value = null;

		#endregion Fields

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the SharedStateItem class.
		/// </summary>
		/// <param name="key">
		/// The key used to access the item. If null a key of string.Empty is used.
		/// Items without a key cannot be accessed by key, only index.
		/// </param>
		/// <param name="type">
		/// The type of object being stored.
		/// </param>
		/// <param name="value">
		/// The value to initialize to.
		/// </param>
		public SharedStateItem(string key, Type type, object value)
		{
			if (type == null)
			{
				throw new ArgumentNullException("type");
			}

			if (key == null)
			{
				key = string.Empty;
			}

			_type = type;
			_key = key;

			if (value == null)
			{
				SetDefaultValue();
			}
			else
			{
				// Use property Set for type validation
				this.Value = value;
			}
		}

		/// <summary>
		/// Initializes a new instance of the SharedStateItem class.
		/// </summary>
		/// <param name="key">
		/// The key used to access the item. If null a key of string.Empty is used.
		/// Items without a key cannot be accessed by key, only index.
		/// </param>
		/// <param name="type">
		/// The type of object being stored.
		/// </param>
		public SharedStateItem(string key, Type type) : this(null, type, null)
		{
		}

		/// <summary>
		/// Initializes a new instance of the SharedStateItem class.
		/// </summary>
		/// <param name="type">
		/// The type of object being stored.
		/// </param>
		public SharedStateItem(Type type) : this(null, type)
		{
		} 

		#endregion Constructors

		#region Properties

		/// <summary>
		/// Gets the items key. Keys are case-sensitive. Returns string.Empty
		/// if not specified.
		/// </summary>
		public string Key
		{
			get
			{
				return _key;
			}
		}

		/// <summary>
		/// Gets the type of object being stored. This is used to determine the
		/// value type when deserialized.
		/// </summary>
		public Type Type
		{
			get
			{
				return _type;
			}
		}

		/// <summary>
		/// The value of the object being stored. If the type of the value does not 
		/// match the current type it is automatically converted. If it cannot be convert 
		/// an exception will be thrown.
		/// <para />
		/// N.B. Null values may be stored against Value when Type represents a value type.
		/// </summary>
		public object Value
		{
			get
			{
				return _value;
			}
			set
			{
				if ((value != null) && (value.GetType() != this.Type))
				{
					value = Convert.ChangeType(value, this.Type);
				}

				_value = value;
			}
		} 

		#endregion Properties

		#region Implementation Methods

		/// <summary>
		/// Sets the default value based on the items type. Reference types are set to null,
		/// Value type to there initial state.
		/// </summary>
		private void SetDefaultValue()
		{
			if (this.Type.IsValueType)
			{
				object defaultValue = Activator.CreateInstance(this.Type);
				this.Value = defaultValue;
			}
			else
			{
				this.Value = null;
			}
		} 

		#endregion Implementation Methods

	}

}
