using System;
using System.Collections;
using System.Data;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Drawing;

namespace RussellMason.Web.UI
{

	/// <summary>
	/// Base web page for outputting details for ViewState and ControlState to the page trace.
	/// </summary>
	public class StateTracePage : WebPage
	{

		#region Fields

		private DataTable _stateTraceTable = null;
		private string _traceError = null;
		private bool _rendered = false;

		#endregion Fields

		#region Implementation Methods

		/// <summary>
		/// Intercepts the writing of state and outputs the content to the trace if enabled.
		/// In derived classes if you do not wish to call base, e.g. you are saving state to
		/// another medium, just call WriteStateTrace directly.
		/// </summary>
		/// <param name="viewState">
		/// The state to output.
		/// </param>
		protected override void SavePageStateToPersistenceMedium(object viewState)
		{
			base.SavePageStateToPersistenceMedium(viewState);

			WriteStateTrace(viewState);
		}

		/// <summary>
		/// Writes the trace if enabled.
		/// </summary>
		protected void WriteStateTrace(object viewState)
		{
			if (this.Trace.IsEnabled)
			{
				try
				{
					_traceError = null;

					CreateTraceTable();
					ExtractState(this.Page, viewState, StateType.ViewState);
				}
				catch (Exception ex)
				{
					_traceError = ex.Message;
				}
			}
		}

		/// <summary>
		/// Extracts the specified state associating it with the control.
		/// As the hierarchy is parsed this method is called recursively.
		/// </summary>
		/// <param name="control">
		/// The control the view state is to be associated with.
		/// </param>
		/// <param name="state">
		/// The current state being examined.
		/// </param>
		/// <param name="stateType">
		/// The type of state (view or control).
		/// </param>
		private void ExtractState(Control control, object state, StateType stateType)
		{
			if (state is Pair)
			{
				ExtractState(control, (Pair) state, stateType);
			}
		}

		/// <summary>
		/// Extracts the specified state associating it with the control.
		/// As the hierarchy is parsed this method is called recursively.
		/// </summary>
		/// <param name="control">
		/// The control the view state is to be associated with.
		/// </param>
		/// <param name="state">
		/// The current state being examined.
		/// </param>
		/// <param name="stateType">
		/// The type of state (view or control).
		/// </param>
		private void ExtractState(Control control, Pair state, StateType stateType)
		{
			if (state.First is object[])
			{
				ExtractState(control, (object[]) state.First, StateType.ControlState);
			}

			if (state.First is ArrayList)
			{
				ExtractStateValues(control, (ArrayList) state.First, stateType);
			}

			if (state.Second is ArrayList)
			{
				ExtractState(control, (ArrayList) state.Second, stateType);
			}

			if (state.First is Pair)
			{
				ExtractState(control, (Pair) state.First, stateType);
			}

			if (state.Second is Pair)
			{
				ExtractState(control, (Pair) state.Second, stateType);
			}
		}

		/// <summary>
		/// Extracts the specified state associating it with the control.
		/// As the hierarchy is parsed this method is called recursively.
		/// </summary>
		/// <param name="control">
		/// The control the view state is to be associated with.
		/// </param>
		/// <param name="state">
		/// The current state being examined.
		/// </param>
		/// <param name="stateType">
		/// The type of state (view or control).
		/// </param>
		private void ExtractState(Control control, object[] state, StateType stateType)
		{
			for (int index = 0; index < state.Length; index += 2)
			{
				object childState = state[index];

				if (childState != null)
				{
					ExtractState(control, childState, stateType);
				}
			}
		}

		/// <summary>
		/// Extracts the specified state associating it with the control.
		/// As the hierarchy is parsed this method is called recursively.
		/// </summary>
		/// <param name="control">
		/// The control the view state is to be associated with.
		/// </param>
		/// <param name="state">
		/// The current state being examined.
		/// </param>
		/// <param name="stateType">
		/// The type of state (view or control).
		/// </param>
		private void ExtractState(Control control, ArrayList state, StateType stateType)
		{
//			if ((state.Count > 0) && (state[0] is IndexedString))
//			{
//				ExtractStateValues(control, state, stateType);
//			}
//			else
			{
				for (int index = 0; index < state.Count; index += 2)
				{
					int childControlIndex = Convert.ToInt32(state[index]);
					object childState = state[index + 1];

					Control childControl = control.Controls[childControlIndex];

					ExtractState(childControl, childState, stateType);
				}
			}
		}

		/// <summary>
		/// Extracts the values of the state associating it with the control.
		/// </summary>
		/// <param name="control">
		/// The control the view state is to be associated with.
		/// </param>
		/// <param name="state">
		/// The state to add to the trace.
		/// </param>
		/// <param name="stateType">
		/// The type of state (view or control).
		/// </param>
		private void ExtractStateValues(Control control, ArrayList state, StateType stateType)
		{
			for (int index = 0; index < state.Count; index += 2)
			{
				object key = state[index];
				object value = state[index + 1];

				AddTraceEntry(control, key, value, stateType);
			}
		}

		/// <summary>
		/// Adds a state entry to the state table.
		/// </summary>
		/// <param name="control">
		/// The control the state is associated with.
		/// </param>
		/// <param name="key">
		/// The key used to access the state.
		/// </param>
		/// <param name="value">
		/// The value of the state.
		/// </param>
		/// <param name="stateType">
		/// The type of state being added.
		/// </param>
		private void AddTraceEntry(Control control, object key, object value, StateType stateType)
		{
			string entry = CreateTraceEnrtyText(key, value);

			DataRow row = _stateTraceTable.NewRow();
			row["Control ID"] = control.UniqueID;
			row["State"] = entry;
			row["StateType"] = (int) stateType;

			_stateTraceTable.Rows.Add(row);
		}

		/// <summary>
		/// Creates a text representation of the value (and any child objects) for output to the trace.
		/// </summary>
		/// <param name="key">
		/// The key used to access the state.
		/// </param>
		/// <param name="value">
		/// The value to trace.
		/// </param>
		/// <returns>
		/// A text representation of the value.
		/// </returns>
		private string CreateTraceEnrtyText(object key, object value)
		{
			string stateKey = string.Empty;
			string stateValue = (value == null) ? string.Empty : value.ToString();

			if (key is string)
			{
				stateKey = (string) key;
			}
//			else if (key is IndexedString)
//			{
//				stateKey = ((IndexedString) key).Value;
//			}

			string quote = string.Empty;

			if (value is string)
			{
				quote = "\"";
			}

			string entry = string.Format("[\"{1}\"] = {0}{2}{0}", quote, stateKey, stateValue);

			entry += CreateTraceEntrySubText(value, 0);

			return entry;
		}

		/// <summary>
		/// Creates a text representation of the value (and any child objects) for output to the trace.
		/// This is used to obtain details from child arrays etc.
		/// </summary>
		/// <param name="value">
		/// The value to trace.
		/// </param>
		/// <param name="level">
		/// The level of indentation.
		/// </param>
		/// <returns>
		/// A text representation of the value.
		/// </returns>
		private string CreateTraceEntrySubText(object value, int level)
		{
			string entry = string.Empty;

			if ((value is IEnumerable) && (!(value is string)))
			{
				string indent = CreateIndent(level);

				int loopIndex = 0;

				foreach (object item in ((IEnumerable) value))
				{
					string quote = string.Empty;

					if (item is string)
					{
						quote = "\"";
					}

					entry += string.Format("<br/>{0}[{2}] = {1}{3}{1}", indent, quote, loopIndex, item.ToString());

					if ((item is IEnumerable) && (!(item is string)))
					{
						entry += CreateTraceEntrySubText(item, ++level);
					}

					loopIndex++;
				}
			}

			return entry;
		}

		/// <summary>
		/// Creates the structure of the table used to store the dicovered state values.
		/// </summary>
		protected void CreateTraceTable()
		{
			_stateTraceTable = new DataTable("State Trace");
			_stateTraceTable.Columns.Add(new DataColumn("Control ID", typeof(string)));
			_stateTraceTable.Columns.Add(new DataColumn("State", typeof(string)));
			_stateTraceTable.Columns.Add(new DataColumn("StateType", typeof(int)));
		}

		/// <summary>
		/// Creates a copy of the state data and removes any rosw that are not of the 
		/// specified state type. Also removes the StateType column so the table can be
		/// directly bound to a grid.
		/// </summary>
		/// <param name="stateType">
		/// The type of state to filter by.
		/// </param>
		/// <returns>
		/// A copy of the state table with only the specified state.
		/// </returns>
		private DataTable CreateTraceTable(StateType stateType)
		{
			const int stateTypeColumnIndex = 2;

			int filter = (int) stateType;

			DataTable table = _stateTraceTable.Copy();

			for (int index = table.Rows.Count - 1; index >= 0; index--)
			{
				DataRow row = table.Rows[index];

				if (Convert.ToInt32(row["StateType"]) != (int) stateType)
				{
					table.Rows.Remove(row);
				}
			}

			table.Columns.RemoveAt(stateTypeColumnIndex);

			return table;
		}

		/// <summary>
		/// Creates an indent to the specified level.
		/// </summary>
		/// <param name="level">
		/// The amount to indent by.
		/// </param>
		/// <returns>
		/// The indent string.
		/// </returns>
		private string CreateIndent(int level)
		{
			string indent = string.Empty;

			for (int index = 0; index < level; index++)
			{
				indent += "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
			}

			return indent;
		}

		/// <summary>
		/// Renders the trace to the page output if enabled.
		/// </summary>
		/// <param name="writer">
		/// The writer to write the trace to.
		/// </param>
		protected override void Render(System.Web.UI.HtmlTextWriter writer)
		{
			base.Render(writer);

			if ((this.Trace.IsEnabled) && (!_rendered))
			{
				Label titleLabel = new Label();
				titleLabel.Width = Unit.Percentage(100);
				titleLabel.Height = new Unit(37);
				titleLabel.BackColor = Color.Black;
				titleLabel.ForeColor = Color.White;
				titleLabel.Font.Name = "Verdana";
				titleLabel.Font.Size = FontUnit.Point(12);
				titleLabel.Font.Bold = true;
				titleLabel.Attributes.Add("style", "padding: 3 3 2 3");

				LiteralControl breakLiteral = new LiteralControl("<br />");

				if (_traceError == null)
				{
					DataGrid viewStateGrid = new DataGrid();
					viewStateGrid.AutoGenerateColumns = true;
					viewStateGrid.BorderWidth = new Unit(0);

					viewStateGrid.Width = Unit.Percentage(100);
					viewStateGrid.CellPadding = 0;
					viewStateGrid.CellSpacing = 0;

					viewStateGrid.HeaderStyle.Font.Name = "Verdana";
					viewStateGrid.HeaderStyle.Font.Size = FontUnit.Point(9);
					viewStateGrid.HeaderStyle.Font.Bold = true;
					viewStateGrid.HeaderStyle.BackColor = Color.FromArgb(0xcccccc);

					viewStateGrid.ItemStyle.Font.Name = "Verdana";
					viewStateGrid.ItemStyle.Font.Size = FontUnit.Point(9);
					viewStateGrid.ItemStyle.BackColor = Color.FromArgb(0xFFFFFF);
					viewStateGrid.ItemStyle.VerticalAlign = VerticalAlign.Top;
					viewStateGrid.ItemStyle.Height = new Unit(16);

					viewStateGrid.AlternatingItemStyle.Font.Name = "Verdana";
					viewStateGrid.AlternatingItemStyle.Font.Size = FontUnit.Point(9);
					viewStateGrid.AlternatingItemStyle.BackColor = Color.FromArgb(0xEEEEEE);
					viewStateGrid.AlternatingItemStyle.VerticalAlign = VerticalAlign.Top;
					viewStateGrid.ItemStyle.Height = new Unit(16);

					// View State

					DataTable viewStateTable = CreateTraceTable(StateType.ViewState);

					titleLabel.Text = "ViewState Content";
					titleLabel.RenderControl(writer);

					viewStateGrid.DataSource = viewStateTable;
					viewStateGrid.DataBind();

					viewStateGrid.RenderControl(writer);

					breakLiteral.RenderControl(writer);

					// Control State

					DataTable controlStateTable = CreateTraceTable(StateType.ControlState);

					titleLabel.Text = "ControlState Content";
					titleLabel.RenderControl(writer);

					viewStateGrid.DataSource = controlStateTable;
					viewStateGrid.DataBind();

					viewStateGrid.RenderControl(writer);
				}
				else
				{
					titleLabel.Text = "State Parse Error";
					titleLabel.RenderControl(writer);

					HtmlGenericControl errorLabel = new HtmlGenericControl("div");
					errorLabel.InnerText = _traceError;
					errorLabel.Style.Add("width", "100%");
					errorLabel.Style.Add("font-family", "Verdana");
					errorLabel.Style.Add("font-size", FontUnit.Point(9).ToString());

					errorLabel.RenderControl(writer);
				}

				breakLiteral.RenderControl(writer);

				_rendered = true;
			}
		}

		#endregion Implementation Methods

	}

}
